import logging
import sys
from argparse import ArgumentParser
from pathlib import Path

from . import server


parser = ArgumentParser(
    description="Run a service to receive file uploads",
)
parser.add_argument(
    "-H",
    "--host",
    default="127.0.0.1",
    help="bind socket to this host (default: %(default)s)",
)
parser.add_argument(
    "-P",
    "--port",
    type=int,
    default=8000,
    help="bind socket to this port (default: %(default)s)",
)
parser.add_argument(
    "-d",
    "--dir",
    type=Path,
    default=Path("."),
    help="save uploads here (default: %(default)s)",
)
parser.add_argument(
    "--overwrite-duplicates",
    action="store_true",
    help="If a file with the same name exists, replace it with the upload. " \
        "When this option is not present, duplicate files are saved with an " \
        "(i) suffix.",
)


def setup_logging():
    formatter = logging.Formatter('%(asctime)s %(message)s', '%H:%M:%S')
    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(logging.INFO)
    handler.setFormatter(formatter)
    server.logger.setLevel(logging.INFO)
    server.logger.addHandler(handler)


def main():
    import os
    from aiohttp.web import run_app

    setup_logging()

    args = parser.parse_args()
    os.chdir(args.dir)
    app = server.make_app(overwrite_duplicates=args.overwrite_duplicates)

    run_app(app, host=args.host, port=args.port)



import setuptools

with open("README.md") as f:
    long_description = f.read()

setuptools.setup(
    name="drope",
    version="0.1.2",
    description="A utility for receiving file uploads",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jvstme/drope",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Utilities",
        "Topic :: Communications :: File Sharing",
    ],

    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",

    entry_points={
        "console_scripts": [
            "drope=drope.cmdline:main",
        ],
    },

    install_requires=[
        "aiofiles ~= 0.8",
        "aiohttp ~= 3.8",
    ],

    extras_require={
        "dev": [
            "pytest",
            "pytest-aiohttp",
        ]
    },

    package_data={
        "": ["static/*"],
    },
    zip_safe=False,
)


import logging
from itertools import count
from pathlib import Path

import aiofiles
import aiofiles.os
from aiohttp import web
from aiohttp.multipart import BodyPartReader


CHUNK_SIZE = 2 ** 16
static_path = Path(__file__).parent.joinpath("static").absolute()

logger = logging.getLogger(__name__)
routes = web.RouteTableDef()


async def unique_filename(original_path: str):
    if not await aiofiles.os.path.exists(original_path):
        return original_path
    
    split = original_path.rsplit(".", 1)
    base = split[0]
    ext = "." + split[1] if len(split) > 1 else ""

    for i in count(1):
        path = f"{base}({i}){ext}"

        if not await aiofiles.os.path.exists(path):
            return path


async def write_file(field: BodyPartReader, filename: str):
    async with aiofiles.open(filename, "wb") as f:
        chunk = await field.read_chunk(size=CHUNK_SIZE)

        while chunk:
            await f.write(chunk)
            chunk = await field.read_chunk(size=CHUNK_SIZE)


@routes.post("/")
async def post_index(request: web.Request):
    try:
        fields = await request.multipart()
    except (ValueError, AssertionError):
        raise web.HTTPBadRequest(
            text="multipart/form-data expected in request body",
        )

    files_received = 0

    async for field in fields:
        if field.name != "file":
            continue

        filename = Path(field.filename or "").name

        if not filename:
            continue

        logger.info("Receiving %s", filename)

        part_filename = filename + ".part"

        if not request.app["overwrite_duplicates"]:
            part_filename = await unique_filename(part_filename)
        
        await write_file(field, part_filename)

        if not request.app["overwrite_duplicates"]:
            filename = await unique_filename(filename)
        elif await aiofiles.os.path.exists(filename):
            await aiofiles.os.remove(filename)
        
        await aiofiles.os.rename(part_filename, filename)

        logger.info("Completed %s", filename)
            
        files_received += 1
    
    if files_received:
        raise web.HTTPSeeOther(".")
    else:
        raise web.HTTPBadRequest(text="file field is required")


@routes.get("/")
async def get_index(_):
    return web.FileResponse(static_path / "index.html")


routes.static("/", static_path)


def make_app(*, overwrite_duplicates=False):
    app = web.Application()
    app.add_routes(routes)

    app["overwrite_duplicates"] = overwrite_duplicates

    return app
